#! perl -w

## Filename:	no_lexicon.pl
## Author:		Sofia Strmbergsson
## Date:		000405
##
## Description:	Lexicon used by Norwegian parser.


###################################################
# LANGUAGE INDEPENDENT LEXICON
###################################################
$HEAD_CHARS = '[a-zA-EG-JL-Z\"\#\$\'\.7-9\?\@\^\_\*]';
$TAIL_CHARS = '[!-+0-6:<>FK\\{}\`~]';
$DIACS_STRUCTURE = '(%[^"]|%"[^"]+")';


###################################################
# LANGUAGE SPECIFIC LEXICON
###################################################
$NOR_SYMBOL = 'A&I|E&U0|E\+&Y|E&I|O&I|i:|I|y:|Y|e\+:||e:|E\+|E|\@:|A:|A|u0:|u:|U0|U|o:|O|'.
		 'p|b|t&S|t\+|t|d\+|d|k|g|m|nK|f|V|s|S|r6|C|h|j|l\+%\)|l\+|l%\)|l|n\+%\)|n\+|n%\)|n|2\'|\'2|\'|\"|\_|\.';
$NOR_VOWEL = 'A&I|E&U0|E\+&Y|E&I|O&I|i:|I|y:|Y|e\+:|e:|E\+|E|\@:|A:|A|u0:|u:|U0|U|o:|O';
$NOR_LONG_VOWEL = 'i:|y:|e\+:|e:|\@:|A:|u0:|u:|o:';
$NOR_SHORT_VOWEL = 'I|Y|E\+|E|A|U|U0|O';
$NOR_DIPHTHONG = 'A&I|E&U0|E\+&Y|E&I|O&I';				# if none, replace with 'NULL'
$NOR_CONSONANT = 'p|b|t&S|t\+|t|d\+|d|k|g|m|nK|f|V|s|S|r6|C|h|j|l\+%\)|l\+|l%\)|l|n\+%\)|n\+|n%\)|n';
$NOR_NOT_SOUND = '2|\'|\.|\_|\"';
$NOR_MAIN_STRESS_MARK = '2[\'\"]|[\'\"]';				# if none, replace with 'NULL'
$NOR_SEC_STRESS_MARK = '\'2';					# if none, replace with 'NULL'
$NOR_SYLLABLE_DELIMITER = '\.|\_';
$NOR_WORD_DELIMITER = '\_';
$NOR_SYLLABIFIED_CONS = 'l\+%\)|l%\)|n\+%\)|n%\)';			# if none, replace with 'NULL'
$NOR_COMPOUND_MARKER = '\+|\_|\s|\-';				# in morphological decomposition string

# if none, replace with 'NULL'
$NOR_TONEME1 = "([^\']+($NOR_SYLLABLE_DELIMITER)|)\'[^\'2]+(\.\'2)?[^2\']+";	# one main stress marking and one secondary stress marking
$NOR_TONEME2 = "([^\']+($NOR_SYLLABLE_DELIMITER)|)2\'[^2\']+(\.\'2)?[^2\']+";	# one main stress marking and one secondary stress marking
$NOR_PHRASE_ACCENT = "(([^\'\"]+($NOR_SYLLABLE_DELIMITER)|)2\"[^2\'\"]+(\.\'2)?[^2\'\"]+)|(([^\'\"]+($NOR_SYLLABLE_DELIMITER)|)\"[^\'2\"]+)";


# patterns where allowed tonal patterns may be overruled
# if the trc string matches the pattern X, the conditions Y, Z and/or W must be fulfilled, where 
# Y = morphological condition
# Z = condition for expansion field
# W = condition on pos
# AND = all conditions must be fulfilled, OR = at least one of the conditions must be fulfilled
# secondary stress preceding main stress allowed in acronyms (words consisting of only capitals), derivations of acronyms and in proper names
$NOR_OK_STRESS_PATTERN{"([^\']+($NOR_SYLLABLE_DELIMITER)|)\'2[^2\']+(\.2?\')[^2\']+"} = 'OR;[A-Z]{2,};.+;npff|npfm|npfn';
# more than one marking of secondary stress allowed in compounds (and some derived words)
$NOR_OK_STRESS_PATTERN{"([^\']+($NOR_SYLLABLE_DELIMITER)|)2?\'([^2\']+(\.\'2?)?[^2\']+)+"} = 'AND;\+;.*;.*';



# Violations generate error messages:
#$NOR_X_CANNOT_FOLLOW_Y{'Y'} = 'X';
#$NOR_X_SHOULD_NOT_FOLLOW_Y{'()()(X1)()'} = '(X2)()(X3)';		X1, X2 and X3 will be printed in error message

# Violations generate warning messages:
#$NOR_X_SHOULD_NOT_FOLLOW_Y{Y} = X;
#$NOR_X_SHOULD_NOT_FOLLOW_Y{()()(X1)()} = (X2)()(X3);		X1, X2 and X3 will be printed in warning message
$NOR_X_SHOULD_NOT_FOLLOW_Y{"()()(r)()"} = '()()([nldt]\+?)';						# ej r fre retroflexkandidat
$NOR_X_SHOULD_NOT_FOLLOW_Y{'()()([lndt]\+)()'} = '()('.$NOR_NOT_SOUND.')*([lndt])[^\+]';				# ej icke-retroflex retroflexkandidat efter retroflex
# ej /e/  i betonad stavelse fre /r/ eller retroflex (ska vara /E/ eller /e:/)
$NOR_X_SHOULD_NOT_FOLLOW_Y{"()()($NOR_MAIN_STRESS_MARK|$NOR_SEC_STRESS_MARK)($NOR_CONSONANT)*"} = '(e)('.$NOR_NOT_SOUND.')*(r|[lnsdt]\+)';
$NOR_X_SHOULD_NOT_FOLLOW_Y{"()()([^dkbrst\.])()"} = '()()(n%\))';						# ej n%) efter ngot som inte r [dlnrst.]
$NOR_X_SHOULD_NOT_FOLLOW_Y{"()()([^dt\.])()"} = '()()(l%\))';						# ej l%) efter ngot som inte r [dt.]
$NOR_X_SHOULD_NOT_FOLLOW_Y{"()()(S)()"} = '()()(pr|tr|kr|V|m|n|p|t|k)';						# ej S fre (V, m, n, p, t, k, pr, tr, kr)


#$TRC_X_CORRESPONDS_TO_ORTO_Y{Y} = X;
# in a X-pattern such as /(X)(Y)(Z)/, extract X and Z
$TRC_X_CORRESPONDS_TO_ORTO_Y{'i(ng)(enir)()'} = "(n)($NOR_NOT_SOUND)+(S)";				# <ng> -> /nS/ i <ingenir>
$TRC_X_CORRESPONDS_TO_ORTO_Y{'arra(ng)(r)()'} = "(nK)($NOR_NOT_SOUND)+(S)";				# <ng> -> /nKS/ i <arrangr>
# <n+g> -> /nKg/ fre sdant som inte r <i>, <j> eller <y>, eftersom <g> d kan uttalas /j/
$TRC_X_CORRESPONDS_TO_ORTO_Y{'(n\+g)()([^ijy])'} = "(nK)($NOR_NOT_SOUND)+(g)";
# <n+k> -> /nKk/ fre sdant som inte r <i>, <j> eller <y>, eftersom <k> d kan uttalas /C/		
$TRC_X_CORRESPONDS_TO_ORTO_Y{'[^rvg](n\+k)([^ijy])()'} = "(nK)($NOR_NOT_SOUND)*(k)";				# <n+?k> -> /nK($NOR_NOT_SOUND)*k/
$TRC_X_CORRESPONDS_TO_ORTO_Y{"([bfgkmprv])(\\+)(\\1)"} = "($NOR_CONSONANT)($NOR_NOT_SOUND)+(\\1)";		# <C+C> -> /C($NOR_NOT_SOUND)+C/
$TRC_X_CORRESPONDS_TO_ORTO_Y{"([dlnt])(\\+)(\\1)"} = "($NOR_CONSONANT)\+?($NOR_NOT_SOUND)+(\\1)";		# <C+C> -> /C(+)?($NOR_NOT_SOUND)+C/
$TRC_X_CORRESPONDS_TO_ORTO_Y{'(s)()(\+s)'} = "(s|S)($NOR_NOT_SOUND)+(s|S)";				# <s+s> -> /(s|S)($NOR_NOT_SOUND)+(s|S)/
$TRC_X_CORRESPONDS_TO_ORTO_Y{'()(\+)(sl)'} = "(S)()(l)";						# <+sl> -> /(S)()(l)/
$TRC_X_CORRESPONDS_TO_ORTO_Y{'()([^\+])(sl)'} = "(s)()(l)";						# <sl> ej i brjan av morfemgrns -> /(s)()(l)/
# <+s+> -> /(s|S)($NOR_NOT_SOUND)+/, i.e. linking /s/ to the left of the syllable boundary
$TRC_X_CORRESPONDS_TO_ORTO_Y{'()()(\+s\+)'} = "(s|S)()($NOR_NOT_SOUND)+";					


# $NOR_TRC_IFF_LANG{TRC} = LANG
# if a trc-sequence should only be found in entries where the language code is equal to LANG
$NOR_TRC_IFF_LANG{"Vst($NOR_SYLLABLE_DELIMITER|\$)"} = "(N|n)no.NOR";
$NOR_TRC_IFF_LANG{"lpt($NOR_SYLLABLE_DELIMITER|\$)"} = "(N|n)no.NOR";
$NOR_TRC_IFF_LANG{"gn($NOR_SYLLABLE_DELIMITER|\$)"} = "(N|n)no.NOR";
$NOR_TRC_IFF_LANG{"kn($NOR_SYLLABLE_DELIMITER|\$)"} = "(N|n)no.NOR";
$NOR_TRC_IFF_LANG{"md($NOR_SYLLABLE_DELIMITER|\$)"} = "(N|n)no.NOR";
$NOR_TRC_IFF_LANG{"mnt($NOR_SYLLABLE_DELIMITER|\$)"} = "(N|n)no.NOR";


# exeptions to the rule that duplication probably is unwanted
$NOR_ALLOWED_DUPLICATION = '[\.][kg]?nn%\)|\.ll%\)';		# if none, replace with ''

# if none, replace with 'NULL'
$NOR_RARE_ONSET = '^(skl|ps|dj|sf)$';

# if none, replace with 'NULL'
$NOR_NOT_INITIAL_ONSET = '^[dtln]\+|sl';

# NB! Rare codas should NOT be included!
$NOR_LEGAL_ONSET = '^(br6|bj|bl|b|C|dr6|d\+|dV|d|fr6|fj|fl|fn|f|gr6|gl|gn|g|h|j|kr6|kl|kn|kV|k|l\+|l|mj|m|n\+|nj|n|pr6|pj|pl|p|r6|skr6|spr6|str6|spj|spl|'.
		'stj|skV|st|sV|sl|sm|sn|sk|sp|s|Skr6|Spr6|Str6|Sk|Sl|Sm|Sn|Spl|Sp|St|SV|S|tr6|t&S|t\+|tj|tV|t|Vr6|V)$';

# if none, replace with 'NULL'
$NOR_RARE_CODA = 'NULL';

$NOR_LEGAL_CODA = 'NULL';

#SOFIAS ORIGINAL:
#$NOR_LEGAL_CODA = 	'^(|[^hCj]|'.				# empty string or any single character except h, C and j
#		'r6|Vd|nKd|nKl|sp|lj|nS|n\+l\+|nKS|t&S|'.		# -NULL
#		'(Vnd|mnd)s?|'.			# -s (3 member clusters)
#		'(mp|r6g|Vn|ft|r6p|lp|ld|Vl|gd|ln|mb|mf|pt)s?|'.	# -s (2 member clusters)
#		'(lf|lV|r6b|r6m|kt|gl|Sk|r6k|mn|nKn|nd)s?|'.	# -s (2 member clusters)
#		'(b|d|t\+|d\+|n\+|l\+)s?|'.			# -s (1 member "clusters")
#		'(t\+|d\+|l\+)S?|'.			# -S
#		'n\+(S?(t\+)?)?|'.			# -S, -St, -t+
#		'(nKg|r6k|Vn|r6m|r6p)t?|'.			# -t
#		'S(k?t?)?|'.				# -t, -kt, -k
#		'm(ts?|s(k|t)?)?|'.			# -t, -s, -ts, -sk, -st
#		'l(ts?|s(kt?|t)?)?|'.			# -t, -s, -ts, -sk, -st, -skt
#		'n(s?ts?|sk?)?|'.			# -s, -t, -sk, -st, -sts, -ts
#		's(ts?|kt?)?|'.			# -t, -ts, -k, -kt
#		'(r6f|f|nKk)(s|ts?)?|'.			# -s, -t, -ts
#		'k(st?|ts?)?|'.			# -s, -t, -ts, -st
#		't(sk?)?|'.				# -s, -sk
#		'lm(s|skt?)?|'.
#		'(V|g|nK)(s?t?)?|'.			# -s, -st, -t
#		'p(sk?|s?t)?|'.			# -s, -t, -sk, -st
#		'(l?g|lk|r6V)(s|t)?|'.			# -s, -t
#		'lpt|gn|kn|md|nmt|Vst)$';			# tilltna kodor i nynorsk



#GISLES ENDRIGER 1/10 2000
$NOR_LEGAL_CODA = 	'^(|[^hCj]|'.				# empty string or any single character except h, C and j
		'nKg|r6|Vd|nKd|nKl|sp|lj|nS|n\+l\+|nKS|t&S|'.		# -NULL
		'(Vnd|mnd|nK)s?|'.			# -s (3 member clusters)
		'(mp|r6g|Vn|ft|r6p|lp|ld|Vl|gd|ln|mb|mf|pt)s?|'.	# -s (2 member clusters)
		'(lf|lV|r6b|r6m|kt|gl|Sk|r6k|mn|nKn|nd|nst)s?|'.	# -s (2 member clusters)
		'(b|d|t\+|d\+|n\+|l\+)s?|'.			# -s (1 member "clusters")
		'(t\+|d\+|l\+)S?|'.			# -S
		'n\+(S?(t\+)?)?|'.			# -S, -St, -t+
		'(fs|vs|r6k|Vn|r6m|r6p|nK)t?|'.			# -t ENDRET HER
		'S(k?t?)?|'.				# -t, -kt, -k
		'm(ts?|s(k|t)?)?|'.			# -t, -s, -ts, -sk, -st
		'l(ts?|s(kt?|t)?)?|'.			# -t, -s, -ts, -sk, -st, -skt
		'n(s?ts?|sk?)?|'.			# -s, -t, -sk, -st, -sts, -ts
		's(ts?|kt?)?|'.			# -t, -ts, -k, -kt
		'(r6f|f|nKk)(s|ts?)?|'.			# -s, -t, -ts
		'k(st?|ts?)?|'.			# -s, -t, -ts, -st
		't(sk?)?|'.				# -s, -sk
		'lm(s|skt?)?|'.
		'(V|g)(s?t?)?|'.			# -s, -st, -t
		'p(sk?|s?t)?|'.			# -s, -t, -sk, -st
		'(l?g|lk|r6V)(s|t)?|'.			# -s, -t
		'lpt|gn|kn|md|nmt|Vst)$';			# tilltna kodor i nynorsk



$NOR_DERIV = 'ring(en|ar(na)?)?s?$|rig(a|e|t)?$|arta(de?s?|t)?$|ls(a|e|t)?$|'.
	     'verk(en|et)?s?$|aktig(e|a|t)?$|artig(e|a|t)?$|skap(et|en)?s?$|het(en|er)?s?$|'.
	     'sam(me|ma)?s?$|halt(ig(a|e)?)?$|^all|messig(t|a|e)?$|sint(a|e)?$|'.
	     '^bar|bar(t|a|e)?$|gd(e|a)?$|vis(e|a|t)?$|enlig(t)?$|vr(d(e|a)?|t)$|'.
	     'makar(en?|na)?s?$|bo(n|r(na)?)?s?$|^ned|^med|'.
	     'sinnig(t|(e|a)?s?)$|sidig(a|e|t)?$|^sam|^vise';
	     
1;	# return 1 to parser-file